@@ -4,7 +4,7 @@ use crate::cookie_path::CookiePath;
 
 use crate::utils::{is_http_scheme, is_secure};
 use cookie::{Cookie as RawCookie, CookieBuilder as RawCookieBuilder, ParseError};
-use serde::{Deserialize, Serialize};
+use serde_derive::{Deserialize, Serialize};
 use std::borrow::Cow;
 use std::convert::TryFrom;
 use std::fmt;
@@ -17,7 +17,7 @@ pub enum Error {
     /// Cookie had attribute HttpOnly but was received from a request-uri which was not an http
     /// scheme
     NonHttpScheme,
-    /// Cookie did not specify domain but was recevied from non-relative-scheme request-uri from
+    /// Cookie did not specify domain but was received from non-relative-scheme request-uri from
     /// which host could not be determined
     NonRelativeScheme,
     /// Cookie received from a request-uri that does not domain-match
@@ -26,6 +26,7 @@ pub enum Error {
     Expired,
     /// `cookie::Cookie` Parse error
     Parse,
+    #[cfg(feature = "public_suffix")]
     /// Cookie specified a public suffix domain-attribute that does not match the canonicalized
     /// request-uri host
     PublicSuffix,
@@ -49,6 +50,7 @@ impl fmt::Display for Error {
                 Error::DomainMismatch => "request-uri does not domain-match the cookie",
                 Error::Expired => "attempted to utilize an Expired Cookie",
                 Error::Parse => "unable to parse string as cookie::Cookie",
+                #[cfg(feature = "public_suffix")]
                 Error::PublicSuffix => "domain-attribute value is a public suffix",
                 Error::UnspecifiedDomain => "domain-attribute is not specified",
             }
@@ -196,7 +198,7 @@ impl<'a> Cookie<'a> {
             .and_then(|p| CookiePath::parse(p))
             .unwrap_or_else(|| CookiePath::default_path(request_url));
 
-        // per RFC6265, Max-Age takes precendence, then Expires, otherwise is Session
+        // per RFC6265, Max-Age takes precedence, then Expires, otherwise is Session
         // only
         let expires = if let Some(max_age) = raw_cookie.max_age() {
             CookieExpiration::from(max_age)
@@ -206,21 +208,8 @@ impl<'a> Cookie<'a> {
             CookieExpiration::SessionEnd
         };
 
-        // These are all tracked via Cookie, clear from RawCookie
-        let mut builder =
-            RawCookieBuilder::new(raw_cookie.name().to_owned(), raw_cookie.value().to_owned());
-        if let Some(secure) = raw_cookie.secure() {
-            builder = builder.secure(secure);
-        }
-        if let Some(http_only) = raw_cookie.http_only() {
-            builder = builder.http_only(http_only);
-        }
-        if let Some(same_site) = raw_cookie.same_site() {
-            builder = builder.same_site(same_site);
-        }
-
         Ok(Cookie {
-            raw_cookie: builder.finish(),
+            raw_cookie: raw_cookie.clone(),
             path,
             expires,
             domain,
@@ -782,7 +771,7 @@ mod serde_tests {
                 None,
             ),
             json!({
-                "raw_cookie": "cookie2=value2",
+                "raw_cookie": "cookie2=value2; Domain=example.com",
                 "path": ["/foo", false],
                 "domain": { "Suffix": "example.com" },
                 "expires": "SessionEnd"
@@ -797,7 +786,7 @@ mod serde_tests {
                 None,
             ),
             json!({
-                "raw_cookie": "cookie3=value3",
+                "raw_cookie": "cookie3=value3; Path=/foo/bar",
                 "path": ["/foo/bar", true],
                 "domain": { "HostOnly": "foo.example.com" },
                 "expires": "SessionEnd",
@@ -815,7 +804,7 @@ mod serde_tests {
                 None,
             ),
             json!({
-                "raw_cookie": "cookie4=value4",
+                "raw_cookie": "cookie4=value4; Expires=Tue, 11 Aug 2015 16:41:42 GMT",
                 "path": ["/foo", false],
                 "domain": { "HostOnly": "example.com" },
                 "expires": { "AtUtc": at_utc.format(crate::rfc3339_fmt::RFC3339_FORMAT).unwrap().to_string() },
@@ -833,10 +822,13 @@ mod serde_tests {
             CookieExpiration::SessionEnd => unreachable!(),
         };
 
+        let utc_formatted= utc_tm.format(&time::format_description::well_known::Rfc2822).unwrap().to_string().replace("+0000", "GMT");
+        let raw_cookie_value = format!("cookie5=value5; Expires={utc_formatted}");
+
         encode_decode(
             &expires,
             json!({
-                "raw_cookie": "cookie5=value5",
+                "raw_cookie": raw_cookie_value,
                 "path":["/foo", false],
                 "domain": { "HostOnly": "example.com" },
                 "expires": { "AtUtc": utc_tm.format(crate::rfc3339_fmt::RFC3339_FORMAT).unwrap().to_string() },
@@ -862,7 +854,7 @@ mod serde_tests {
         encode_decode(
             &max_age,
             json!({
-                "raw_cookie": "cookie6=value6",
+                "raw_cookie": "cookie6=value6; Max-Age=10; Expires=Tue, 11 Aug 2015 16:41:42 GMT",
                 "path":["/foo", false],
                 "domain": { "HostOnly": "example.com" },
                 "expires": { "AtUtc": utc_tm.format(crate::rfc3339_fmt::RFC3339_FORMAT).unwrap().to_string() },
@@ -882,7 +874,7 @@ mod serde_tests {
         encode_decode(
             &max_age,
             json!({
-                "raw_cookie": "cookie7=value7",
+                "raw_cookie": "cookie7=value7; Max-Age=10",
                 "path":["/foo", false],
                 "domain": { "HostOnly": "example.com" },
                 "expires": { "AtUtc": utc_tm.format(crate::rfc3339_fmt::RFC3339_FORMAT).unwrap().to_string() },
